@@ -30,12 +30,13 @@ fn main() {
     #[cfg(feature = "codec-aom")]
     {
         let include =
-            env::var_os("DEP_AOM_INCLUDE").expect("libaom-sys should have set pkgconfig path");
+            env::var_os("DEP_AOM_INCLUDE").expect("libaom-sys should have set include path");
         avif.define("AVIF_CODEC_AOM", "1");
         avif.define("AOM_INCLUDE_DIR", include);
 
         let pc_path =
             env::var_os("DEP_AOM_PKGCONFIG").expect("libaom-sys should have set pkgconfig path");
+        avif.define("AOM_LIBRARY", pc_path.clone());
         pc_paths.insert(0, pc_path.into());
     }
 
@@ -78,9 +79,24 @@ fn main() {
     eprintln!("pc=\"{:?}\"", local_pc_files);
     avif.env("PKG_CONFIG_PATH", local_pc_files);
 
-    avif.profile("Release")
-        .configure_arg("-DCMAKE_INSTALL_LIBDIR=lib")
-        .configure_arg("-DCMAKE_DISABLE_FIND_PACKAGE_libyuv=1");
+    avif.profile(if env::var("PROFILE").expect("PROFILE") == "release" {
+        "Release"
+    } else {
+        "Debug"
+    })
+    .configure_arg("-DCMAKE_INSTALL_LIBDIR=lib")
+    .configure_arg("-DCMAKE_DISABLE_FIND_PACKAGE_libyuv=1");
+    if env::var("LIBAVIF_CROSS_WIN32").is_ok() {
+        avif.configure_arg("-T host=x64").configure_arg("-A Win32");
+    }
+    if env::var("TARGET").expect("TARGET").contains("wasm") {
+        avif.define(
+            "CMAKE_TOOLCHAIN_FILE",
+            env::var("EMSCRIPTEN_CMAKE_FILE").expect(
+              "EMSCRIPTEN_CMAKE_FILE must be set if you want to build wasm target, it's the local path to https://github.com/emscripten-core/emscripten/blob/main/cmake/Modules/Platform/Emscripten.cmake",
+          ),
+        );
+    }
     let avif_built = avif.build();
 
     println!(
