@@ -3,6 +3,12 @@
 #ifndef RAV1E_H
 #define RAV1E_H
 
+
+#define RAV1E_MAJOR 0
+#define RAV1E_MINOR 5
+#define RAV1E_PATCH 0
+
+
 #include <stddef.h>
 #include <stdint.h>
 #include <stdlib.h>
@@ -66,11 +72,11 @@ typedef enum {
     /**
      * BT.470 System M (historical)
      */
-    RA_COLOR_PRIMARIES_BT470_M = 4,
+    RA_COLOR_PRIMARIES_BT470M = 4,
     /**
      * BT.470 System B, G (historical)
      */
-    RA_COLOR_PRIMARIES_BT470_BG,
+    RA_COLOR_PRIMARIES_BT470BG,
     /**
      * BT.601-7 525 (SMPTE 170 M)
      */
@@ -214,7 +220,7 @@ typedef enum {
     /**
      * BT.470 System B, G (historical)
      */
-    RA_MATRIX_COEFFICIENTS_BT470_BG,
+    RA_MATRIX_COEFFICIENTS_BT470BG,
     /**
      * BT.601-7 525 (SMPTE 170 M)
      */
@@ -226,15 +232,15 @@ typedef enum {
     /**
      * YCgCo
      */
-    RA_MATRIX_COEFFICIENTS_YCG_CO,
+    RA_MATRIX_COEFFICIENTS_Y_CG_CO,
     /**
      * BT.2020 non-constant luminance, BT.2100 YCbCr
      */
-    RA_MATRIX_COEFFICIENTS_BT2020_NCL,
+    RA_MATRIX_COEFFICIENTS_BT2020NCL,
     /**
      * BT.2020 constant luminance
      */
-    RA_MATRIX_COEFFICIENTS_BT2020_CL,
+    RA_MATRIX_COEFFICIENTS_BT2020CL,
     /**
      * SMPTE ST 2085 YDzDx
      */
@@ -250,7 +256,7 @@ typedef enum {
     /**
      * BT.2020 ICtCp
      */
-    RA_MATRIX_COEFFICIENTS_ICT_CP,
+    RA_MATRIX_COEFFICIENTS_I_CT_CP,
 } RaMatrixCoefficients;
 
 /**
@@ -269,6 +275,37 @@ typedef enum {
     RA_PIXEL_RANGE_FULL,
 } RaPixelRange;
 
+/**
+ * Rate Control Data
+ */
+typedef enum {
+    /**
+     * A Rate Control Summary Packet
+     *
+     * It is emitted once, after the encoder is flushed.
+     *
+     * It contains a summary of the rate control information for the
+     * encoding process that just terminated.
+     */
+    RA_RC_DATA_KIND_SUMMARY,
+    /**
+     * A Rate Control Frame-specific Packet
+     *
+     * It is emitted every time a frame is processed.
+     *
+     * The information contained is required to encode its matching
+     * frame in a second pass encoding.
+     */
+    RA_RC_DATA_KIND_FRAME,
+    /**
+     * There is no pass data available for now
+     *
+     * This is emitted if rav1e_rc_receive_pass_data is called more
+     * often than it should.
+     */
+    RA_RC_DATA_KIND_EMPTY,
+} RaRcDataKind;
+
 /**
  * Supported Transfer Characteristics
  *
@@ -286,11 +323,11 @@ typedef enum {
     /**
      * BT.470 System M (historical)
      */
-    RA_TRANSFER_CHARACTERISTICS_BT470_M = 4,
+    RA_TRANSFER_CHARACTERISTICS_BT470M = 4,
     /**
      * BT.470 System B, G (historical)
      */
-    RA_TRANSFER_CHARACTERISTICS_BT470_BG,
+    RA_TRANSFER_CHARACTERISTICS_BT470BG,
     /**
      * BT.601-7 525 (SMPTE 170 M)
      */
@@ -373,35 +410,6 @@ typedef struct RaContext RaContext;
  */
 typedef struct RaFrame RaFrame;
 
-/**
- * Chromaticity coordinates as defined by CIE 1931, expressed as 0.16
- * fixed-point values.
- */
-typedef struct {
-    /**
-     * The X coordinate.
-     */
-    uint16_t x;
-    /**
-     * The Y coordinate.
-     */
-    uint16_t y;
-} RaChromaticityPoint;
-
-/**
- * A rational number.
- */
-typedef struct {
-    /**
-     * Numerator.
-     */
-    uint64_t num;
-    /**
-     * Denominator.
-     */
-    uint64_t den;
-} RaRational;
-
 /**
  * Simple Data
  *
@@ -420,6 +428,37 @@ typedef struct {
     size_t len;
 } RaData;
 
+/**
+ * A rational number.
+ */
+typedef struct {
+    /**
+     * Numerator.
+     */
+    uint64_t num;
+    /**
+     * Denominator.
+     */
+    uint64_t den;
+} RaRational;
+
+/**
+ * Chromaticity coordinates as defined by CIE 1931, expressed as 0.16
+ * fixed-point values.
+ */
+typedef struct {
+    /**
+     * The X coordinate.
+     */
+    uint16_t x;
+    /**
+     * The Y coordinate.
+     */
+    uint16_t y;
+} RaChromaticityPoint;
+
+typedef void (*RaFrameOpaqueCb)(void*);
+
 /**
  * Encoded Packet
  *
@@ -444,37 +483,109 @@ typedef struct {
      * Frame type
      */
     RaFrameType frame_type;
+    /**
+     * User provided opaque data
+     */
+    void *opaque;
+    /**
+     * The reconstruction of the shown frame.
+     * This is freed automatically by rav1e_packet_unref().
+     */
+    RaFrame *rec;
+    /**
+     * The Reference Frame
+     * This is freed automatically by rav1e_packet_unref().
+     */
+    RaFrame *source;
 } RaPacket;
 
 #ifdef __cplusplus
 extern "C" {
 #endif // __cplusplus
 
+/**
+ * Version information as presented in `[package]` `version`.
+ *
+ * e.g. `0.1.0``
+ *
+ * Can be parsed by [semver](https://crates.io/crates/semver).
+ * This returns the version of the loaded library, regardless
+ * of which version the library user was built against.
+ */
+const char *rav1e_version_short(void);
+
+/**
+ * Version information with the information
+ * provided by `git describe --tags`.
+ *
+ * e.g. `0.1.0 (v0.1.0-1-g743d464)`
+ *
+ * This returns the version of the loaded library, regardless
+ * of which version the library user was built against.
+ */
+const char *rav1e_version_full(void);
+
+/**
+ * Free a RaData buffer
+ */
+void rav1e_data_unref(RaData *data);
+
 /**
  * Create a RaConfig filled with default parameters.
  */
 RaConfig *rav1e_config_default(void);
 
 /**
- * Set a configuration parameter using its key and value as string.
+ * Setup a second pass rate control using the provided summary
  *
- * Available keys and values
- * - "quantizer": 0-255, default 100
- * - "speed": 0-10, default 6
- * - "tune": "psnr"-"psychovisual", default "psychovisual"
+ * Passing NULL data resets the rate control settings.
  *
- * Return a negative value on error or 0.
+ * If additional data is required, pointer and len stay unchanged, otherwise
+ * they are updated.
+ *
+ * Return:
+ * 0 on success
+ * > 0 if the buffer has to be larger
+ * < 0 on failure
  */
-int rav1e_config_parse(RaConfig *cfg, const char *key, const char *value);
+int rav1e_config_set_rc_summary(RaConfig *cfg, const uint8_t **data, size_t *len);
 
 /**
- * Set a configuration parameter using its key and value as integer.
+ * Request to emit pass data
  *
- * Available keys and values are the same as rav1e_config_parse()
+ * Set emit to 0 to not emit pass data, non-zero to emit pass data.
  *
- * Return a negative value on error or 0.
  */
-int rav1e_config_parse_int(RaConfig *cfg, const char *key, int value);
+void rav1e_config_set_emit_data(RaConfig *cfg, int emit);
+
+/**
+ * Set the display aspect ratio of the stream
+ *
+ * Needed for anamorphic video.
+ */
+void rav1e_config_set_sample_aspect_ratio(RaConfig *cfg, RaRational sample_aspect_ratio);
+
+/**
+ * Set the time base of the stream
+ *
+ * Needed for rate control.
+ */
+void rav1e_config_set_time_base(RaConfig *cfg, RaRational time_base);
+
+/**
+ * Set pixel format of the stream.
+ *
+ * Supported values for subsampling and chromapos are defined by the
+ * enum types RaChromaSampling and RaChromaSamplePosition respectively.
+ * Valid values for fullrange are 0 and 1.
+ *
+ * Returns a negative value on error or 0.
+ */
+int rav1e_config_set_pixel_format(RaConfig *cfg,
+                                  uint8_t bit_depth,
+                                  RaChromaSampling subsampling,
+                                  RaChromaSamplePosition chroma_pos,
+                                  RaPixelRange pixel_range);
 
 /**
  * Set color properties of the stream.
@@ -510,46 +621,54 @@ int rav1e_config_set_content_light(RaConfig *cfg,
  * Returns a negative value on error or 0.
  */
 int rav1e_config_set_mastering_display(RaConfig *cfg,
-                                       RaChromaticityPoint primaries[3],
+                                       const RaChromaticityPoint primaries[3],
                                        RaChromaticityPoint white_point,
                                        uint32_t max_luminance,
                                        uint32_t min_luminance);
 
-/**
- * Set pixel format of the stream.
- *
- * Supported values for subsampling and chromapos are defined by the
- * enum types RaChromaSampling and RaChromaSamplePosition respectively.
- * Valid values for fullrange are 0 and 1.
- *
- * Returns a negative value on error or 0.
- */
-int rav1e_config_set_pixel_format(RaConfig *cfg,
-                                  uint8_t bit_depth,
-                                  RaChromaSampling subsampling,
-                                  RaChromaSamplePosition chroma_pos,
-                                  RaPixelRange pixel_range);
-
-/**
- * Set the time base of the stream
- *
- * Needed for rate control.
- */
-void rav1e_config_set_time_base(RaConfig *cfg, RaRational time_base);
-
 /**
  * Free the RaConfig.
  */
 void rav1e_config_unref(RaConfig *cfg);
 
 /**
- * Produce a sequence header matching the current encoding context
+ * Set a configuration parameter using its key and value as string.
  *
- * Its format is compatible with the AV1 Matroska and ISOBMFF specification.
+ * Available keys and values
+ * - "width": width of the frame, default 640
+ * - "height": height of the frame, default 480
+ * - "speed": 0-10, default 6
+ * - "threads": maximum number of threads to be used
+ * - "tune": "psnr"-"psychovisual", default "psychovisual"
+ * - "quantizer": 0-255, default 100
+ * - "tiles": total number of tiles desired (0 denotes auto), default 0
+ * - "tile_rows": number of tiles horizontally (must be a power of two, overridden by tiles if present), default 0
+ * - "tile_cols": number of tiles vertically (must be a power of two, overridden by tiles if present), default 0
+ * - "min_quantizer": minimum allowed base quantizer to use in bitrate mode, default 0
+ * - "bitrate": target bitrate for the bitrate mode (required for two pass mode), default 0
+ * - "key_frame_interval": maximum interval between two keyframes, default 240
+ * - "min_key_frame_interval": minimum interval between two keyframes, default 12
+ * - "switch_frame_interval": interval between switch frames, default 0
+ * - "reservoir_frame_delay": number of temporal units over which to distribute the reservoir usage, default None
+ * - "rdo_lookahead_frames": number of frames to read ahead for the RDO lookahead computation, default 40
+ * - "low_latency": flag to enable low latency mode, default false
+ * - "enable_timing_info": flag to enable signaling timing info in the bitstream, default false
+ * - "still_picture": flag for still picture mode, default false
  *
- * Use rav1e_data_unref() to free it.
+ * Return a negative value on error or 0.
  */
-RaData *rav1e_container_sequence_header(const RaContext *ctx);
+int rav1e_config_parse(RaConfig *cfg,
+                       const char *key,
+                       const char *value);
+
+/**
+ * Set a configuration parameter using its key and value as integer.
+ *
+ * Available keys and values are the same as rav1e_config_parse()
+ *
+ * Return a negative value on error or 0.
+ */
+int rav1e_config_parse_int(RaConfig *cfg, const char *key, int value);
 
 /**
  * Generate a new encoding context from a populated encoder configuration
@@ -566,9 +685,189 @@ RaContext *rav1e_context_new(const RaConfig *cfg);
 void rav1e_context_unref(RaContext *ctx);
 
 /**
- * Free a RaData buffer
+ * Produce a new frame from the encoding context
+ *
+ * It must be populated using rav1e_frame_fill_plane().
+ *
+ * The frame is reference counted and must be released passing it to rav1e_frame_unref(),
+ * see rav1e_send_frame().
  */
-void rav1e_data_unref(RaData *data);
+RaFrame *rav1e_frame_new(const RaContext *ctx);
+
+/**
+ * Free the RaFrame.
+ */
+void rav1e_frame_unref(RaFrame *frame);
+
+/**
+ * Overrides the encoders frame type decision for a frame
+ *
+ * Must be called before rav1e_send_frame() if used.
+ */
+int rav1e_frame_set_type(RaFrame *frame, RaFrameTypeOverride frame_type);
+
+/**
+ * Register an opaque data and a destructor to the frame
+ *
+ * It takes the ownership of its memory:
+ * - it will relinquish the ownership to the context if
+ *   rav1e_send_frame is called.
+ * - it will call the destructor if rav1e_frame_unref is called
+ *   otherwise.
+ */
+void rav1e_frame_set_opaque(RaFrame *frame, void *opaque, RaFrameOpaqueCb cb);
+
+/**
+ * Retrieve the first-pass data of a two-pass encode for the frame that was
+ * just encoded. This should be called BEFORE every call to rav1e_receive_packet()
+ * (including the very first one), even if no packet was produced by the
+ * last call to rav1e_receive_packet, if any (i.e., RA_ENCODER_STATUS_ENCODED
+ * was returned). It needs to be called once more after
+ * RA_ENCODER_STATUS_LIMIT_REACHED is returned, to retrieve the header that
+ * should be written to the front of the stats file (overwriting the
+ * placeholder header that was emitted at the start of encoding).
+ *
+ * It is still safe to call this function when rav1e_receive_packet() returns any
+ * other error. It will return NULL instead of returning a duplicate copy
+ * of the previous frame's data.
+ *
+ * Must be freed with rav1e_data_unref().
+ */
+RaData *rav1e_twopass_out(RaContext *ctx);
+
+/**
+ * Return the Rate Control Summary Packet size
+ *
+ * It is useful mainly to preserve space when saving
+ * both Rate Control Summary and Frame Packets in a single file
+ */
+size_t rav1e_rc_summary_size(const RaContext *ctx);
+
+/**
+ * Return the first pass data
+ *
+ * Call it after rav1e_receive_packet() returns a normal condition status:
+ * EncoderStatus::Encoded,
+ * EncoderStatus::Success,
+ * EncoderStatus::LimitReached.
+ *
+ * use rav1e_data_unref() to free the data.
+ *
+ * It will return a `RcDataKind::Summary` once the encoder is flushed.
+ */
+RaRcDataKind rav1e_rc_receive_pass_data(RaContext *ctx, RaData **data);
+
+/**
+ * Number of pass data packets required to progress the encoding process.
+ *
+ * At least that number of packets must be passed before the encoder can
+ * progress.
+ *
+ * Stop feeding-in pass data packets once the function returns 0.
+ *
+ * ``` c
+ * while (rav1e_rc_second_pass_data_required(ctx) > 0) {
+ *   int more = rav1e_rc_send_pass_data(ctx, &data, &len);
+ *   if (more > 0) {
+ *      refill(&data, &len);
+ *   } else if (more < 0) {
+ *     goto fail;
+ *   }
+ * }
+ * ```
+ *
+ */
+int32_t rav1e_rc_second_pass_data_required(const RaContext *ctx);
+
+/**
+ * Feed the first pass Rate Control data to the encoder,
+ * Frame-specific Packets only.
+ *
+ * Call it before receive_packet()
+ *
+ * If additional data is required, pointer and len stay unchanged, otherwise
+ * they are updated.
+ *
+ * Returns:
+ * - `0` on success,
+ * - `> 0` the amount of bytes needed
+ * - `< 0` on unrecoverable failure
+ */
+int rav1e_rc_send_pass_data(RaContext *ctx, const uint8_t **data, size_t *len);
+
+/**
+ * Ask how many bytes of the stats file are needed before the next frame
+ * of the second pass in a two-pass encode can be encoded. This is a lower
+ * bound (more might be required), but if 0 is returned, then encoding can
+ * proceed. This is just a hint to the application, and does not need to
+ * be called for encoding the second pass to work, so long as the
+ * application continues to provide more data to rav1e_twopass_in() in a loop
+ * until rav1e_twopass_in() returns 0.
+ */
+size_t rav1e_twopass_bytes_needed(RaContext *ctx);
+
+/**
+ * Provide stats data produced in the first pass of a two-pass encode to the
+ * second pass. On success this returns the number of bytes of that data
+ * which were consumed. When encoding the second pass of a two-pass encode,
+ * this should be called repeatedly in a loop before every call to
+ * rav1e_receive_packet() (including the very first one) until no bytes are
+ * consumed, or until twopass_bytes_needed() returns 0. Returns -1 on failure.
+ */
+int rav1e_twopass_in(RaContext *ctx, uint8_t *buf, size_t buf_size);
+
+/**
+ * Send the frame for encoding
+ *
+ * The function increases the frame internal reference count and it can be passed multiple
+ * times to different rav1e_send_frame() with a caveat:
+ *
+ * The opaque data, if present, will be moved from the Frame to the context
+ * and returned by rav1e_receive_packet in the Packet opaque field or
+ * the destructor will be called on rav1e_context_unref if the frame is
+ * still pending in the encoder.
+ *
+ * Returns:
+ * - `0` on success,
+ * - `> 0` if the input queue is full
+ * - `< 0` on unrecoverable failure
+ */
+RaEncoderStatus rav1e_send_frame(RaContext *ctx, RaFrame *frame);
+
+/**
+ * Return the last encoder status
+ */
+RaEncoderStatus rav1e_last_status(const RaContext *ctx);
+
+/**
+ * Return a static string matching the EncoderStatus variant.
+ *
+ */
+const char *rav1e_status_to_str(RaEncoderStatus status);
+
+/**
+ * Receive encoded data
+ *
+ * Returns:
+ * - `0` on success
+ * - `> 0` if additional frame data is required
+ * - `< 0` on unrecoverable failure
+ */
+RaEncoderStatus rav1e_receive_packet(RaContext *ctx, RaPacket **pkt);
+
+/**
+ * Free the RaPacket.
+ */
+void rav1e_packet_unref(RaPacket *pkt);
+
+/**
+ * Produce a sequence header matching the current encoding context
+ *
+ * Its format is compatible with the AV1 Matroska and ISOBMFF specification.
+ *
+ * Use rav1e_data_unref() to free it.
+ */
+RaData *rav1e_container_sequence_header(const RaContext *ctx);
 
 /**
  * Fill a frame plane
@@ -593,126 +892,29 @@ void rav1e_frame_fill_plane(RaFrame *frame,
                             int bytewidth);
 
 /**
- * Produce a new frame from the encoding context
+ * Extract a frame plane
  *
- * It must be populated using rav1e_frame_fill_plane().
+ * This is the reverse of rav1e_frame_fill_plane(), primarily used for
+ * extracting the source and reconstruction data from a RaPacket.
  *
- * The frame is reference counted and must be released passing it to rav1e_frame_unref(),
- * see rav1e_send_frame().
- */
-RaFrame *rav1e_frame_new(const RaContext *ctx);
-
-/**
- * Overrides the encoders frame type decision for a frame
+ * Currently the frame contains 3 planes, the first is luminance followed by
+ * chrominance.
  *
- * Must be called before rav1e_send_frame() if used.
- */
-int rav1e_frame_set_type(RaFrame *frame, RaFrameTypeOverride frame_type);
-
-/**
- * Free the RaFrame.
- */
-void rav1e_frame_unref(RaFrame *frame);
-
-/**
- * Return the last encoder status
- */
-RaEncoderStatus rav1e_last_status(const RaContext *ctx);
-
-/**
- * Free the RaPacket.
- */
-void rav1e_packet_unref(RaPacket *pkt);
-
-/**
- * Receive encoded data
+ * The data is copied out of the frame for a single plane.
  *
- * Returns:
- * - `0` on success
- * - `> 0` if additional frame data is required
- * - `< 0` on unrecoverable failure
+ * frame: A frame provided inside a packet returned by rav1e_receive_packet()
+ * plane: The index of the plane starting from 0
+ * data: The destination for the data
+ * data_len: Length of the buffer
+ * stride: Plane line in bytes, including padding
+ * bytewidth: Number of bytes per component, either 1 or 2
  */
-RaEncoderStatus rav1e_receive_packet(RaContext *ctx, RaPacket **pkt);
-
-/**
- * Send the frame for encoding
- *
- * The function increases the frame internal reference count and it can be passed multiple
- * times to different rav1e_send_frame().
- *
- * Returns:
- * - `0` on success,
- * - `> 0` if the input queue is full
- * - `< 0` on unrecoverable failure
- */
-RaEncoderStatus rav1e_send_frame(RaContext *ctx, const RaFrame *frame);
-
-/**
- * Return a static string matching the EncoderStatus variant.
- *
- */
-const char *rav1e_status_to_str(RaEncoderStatus status);
-
-/**
- * Ask how many bytes of the stats file are needed before the next frame
- * of the second pass in a two-pass encode can be encoded. This is a lower
- * bound (more might be required), but if 0 is returned, then encoding can
- * proceed. This is just a hint to the application, and does not need to
- * be called for encoding the second pass to work, so long as the
- * application continues to provide more data to rav1e_twopass_in() in a loop
- * until rav1e_twopass_in() returns 0.
- */
-size_t rav1e_twopass_bytes_needed(RaContext *ctx);
-
-/**
- * Provide stats data produced in the first pass of a two-pass encode to the
- * second pass. On success this returns the number of bytes of that data
- * which were consumed. When encoding the second pass of a two-pass encode,
- * this should be called repeatedly in a loop before every call to
- * rav1e_receive_packet() (including the very first one) until no bytes are
- * consumed, or until twopass_bytes_needed() returns 0. Returns -1 on failure.
- */
-int rav1e_twopass_in(RaContext *ctx, uint8_t *buf, size_t buf_size);
-
-/**
- * Retrieve the first-pass data of a two-pass encode for the frame that was
- * just encoded. This should be called BEFORE every call to rav1e_receive_packet()
- * (including the very first one), even if no packet was produced by the
- * last call to rav1e_receive_packet, if any (i.e., RA_ENCODER_STATUS_ENCODED
- * was returned). It needs to be called once more after
- * RA_ENCODER_STATUS_LIMIT_REACHED is returned, to retrieve the header that
- * should be written to the front of the stats file (overwriting the
- * placeholder header that was emitted at the start of encoding).
- *
- * It is still safe to call this function when rav1e_receive_packet() returns any
- * other error. It will return NULL instead of returning a duplicate copy
- * of the previous frame's data.
- *
- * Must be freed with rav1e_data_unref().
- */
-RaData *rav1e_twopass_out(RaContext *ctx);
-
-/**
- * Version information with the information
- * provided by `git describe --tags`.
- *
- * e.g. `0.1.0 (v0.1.0-1-g743d464)`
- *
- * This returns the version of the loaded library, regardless
- * of which version the library user was built against.
- */
-const char *rav1e_version_full(void);
-
-/**
- * Version information as presented in `[package]` `version`.
- *
- * e.g. `0.1.0``
- *
- * Can be parsed by [semver](https://crates.io/crates/semver).
- * This returns the version of the loaded library, regardless
- * of which version the library user was built against.
- */
-const char *rav1e_version_short(void);
+void rav1e_frame_extract_plane(const RaFrame *frame,
+                               int plane,
+                               uint8_t *data,
+                               size_t data_len,
+                               ptrdiff_t stride,
+                               int bytewidth);
 
 #ifdef __cplusplus
 } // extern "C"
